require 'optparse'
require 'box'

module Box

  #
  class Command

    # Initialize and run command.
    def self.run
      new.run
    end

    #
    attr :options

    #
    def parse
      options = {:types=>[]}

      usage = OptionParser.new do |opt|

        opt.banner = "Usage: box [options] [path]\n" +
                     "Supported Types: zip gz gem"

        #opt.on('-c', '--create', "Create packages. This is default operation.") do
        #  options[:create] = true
        #end

        opt.on('-t', '--type [TYPE]', "Type of package (can be used more than once)") do |type|
          options[:types] << type
        end

        opt.on('-o', '--output [DIR]', "Where to save packages, defaults to current") do |output|
          options[:output] = output
        end

        opt.on('-m', '--manifest [FILE]', "Manifest file, defaults to MANIFEST{,.txt}") do |file|
          options[:manifest] = file
        end

        opt.on('-s', '--spec', "Save type specification to current directory, if applicable") do
          options[:spec] = true
        end

        opt.on('-q', '--quiet', "Surpress any non-essential output") do
          options[:quiet] = true
        end

        opt.on('--trial', "Turn on trial mode (no disk writes)") do
          $TRIAL = true
        end

        opt.on('--debug', "Turn on debug mode") do
          $DEBUG = true
        end

        opt.on_tail('-h', '--help', "Display this help message") do
          puts opt
          exit
        end

      end

      usage.parse!(ARGV)

      @options = options
    end

    def run
      parse
      execute
    end

    def execute
      types = options[:types]
      types = [ENV['BOX_TYPE'] || 'gz'] if types.empty?

      loc = ARGV.shift || Dir.pwd

      begin
        Box.package(loc, types, options)
      rescue => err
        raise err if $DEBUG
        $stderr.puts(err)
      end
    end

  end

end

