require 'box/pack'

module Box

  # = Debian Apt-Get Package
  #
  class Deb < Pack

    def initialize(location, options)
      super(location, options)
      # ...
    end

    def extension ; '.deb' ; end

    # Create a debian package.
    #
    def package_build
      #unless package_needed? or force?
      #  report_package_already_built(package_file)
      #  return
      #end

      return if dryrun?

      report "Building debian package..."
      #status("dpkg -b #{package_file}")      

      file = nil

      stage(extension)

      # mock install
      mock_folder = mock_install(stage_folder)

      ctrl       = POM::DebCtrl.from_pom(project.metadata)
      ctrl_file  = File.join(mock_folder, 'DEBIAN', 'control')

      mkdir_p(File.dirname(ctrl_file)

      File.open(ctrl_file, 'w') { |f| f << ctrl.to_s }

      cd(temp_folder) do
        rm(package_file) if package_file.exist?
        sh %{dpkg-deb -b #{mock_folder} #{package_file}}
      end

      file = File.join(temp_folder, package_file)

      #file = transfer(file, pack_folder)
      #report_package_built(file)

      return file
    end

    # Create debian package.
    def require_debian_tools
      if true
        # TODO: Ensure required debian tools here
      else
        raise "Debian package requested, but debian tools not found."
      end
    end

    # Return the name of the generated package file.

    def package_file
      debian_package_file
    end

    # Install debian package.
    #--
    # TODO: No choice but to shell out installation at this point.
    # Are there debian tools for Ruby?
    #++

    #def package_install
    #  Dir.chdir( work_folder ) do
    #    sh "dpkg --install #{debian_package_file}"
    #  end
    #end

    # Uninstall debian package.

    #def package_uninstall
    #  sh "dpkg --remove lib#{spec.name}"
    #end

  private

    def debian_package_name
      name = metadata.name.downcase.gsub(/\W+/,'').gsub(/_/,'')
      rver = RUBY_VERSION.sub(/[.][0-9]+$/,'')
      "lib#{name}-ruby#{rver}"
    end

    def debian_package_file
      arch = metadata.architecture.downcase
      arch = (arch == 'any' ? 'all' : arch)
      if arch == 'all'
        debian_package_name + "_#{metadata.version}.deb"
      else
        debian_package_name + "_#{metadata.version}_#{arch}.deb"
      end
    end

  end

end

